"""
    Voot Kodi Addon
    Copyright (C) 2018 gujal

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import sys
from kodi_six import xbmcgui, xbmcplugin, xbmcaddon
import requests
import math
try:
    import StorageServer
except:
    import storageserverdummy as StorageServer

# Get the plugin url in plugin:// notation.
_url = sys.argv[0]
# Get the plugin handle as an integer number.
_handle = int(sys.argv[1])

_addon = xbmcaddon.Addon()
_addonname = _addon.getAddonInfo('name')
_version = _addon.getAddonInfo('version')
_addonID = _addon.getAddonInfo('id')
_icon = _addon.getAddonInfo('icon')
_fanart = _addon.getAddonInfo('fanart')
_settings = _addon.getSetting

cache = StorageServer.StorageServer('voot', _settings('timeout'))


def clear_cache():
    """
    Clear the cache database.
    """
    msg = 'Cached Data has been cleared'
    cache.table_name = 'voot'
    cache.cacheDelete('%get%')
    xbmcgui.Dialog().notification(_addonname, msg, _icon, 3000, False)


if _settings('version') != _version:
    _addon.setSetting('version', _version)
    clear_cache()
    _addon.openSettings()

apiUrl = 'https://apiv2.voot.com/wsv_2_3/'
headers = {'User-Agent': 'okhttp/3.4.1'}
if _settings('EnableIP') == 'true':
    headers['X-Forwarded-For'] = _settings('ipaddress')

sortID = 'sortId=mostPopular'
if _settings('tvsort') == 'Name':
    sortID = 'sortId=a_to_z'

msort = 'sortId=mostPopular'
if _settings('msort') == 'Name':
    msort = 'sortId=a_to_z'

qualities = ['Tablet Main', 'TV Main', 'HLS_TV_HD']
strqual = qualities[int(_settings('quality'))]


def get_top():
    """
    Get the list of countries.
    :return: list
    """
    MAINLIST = ['Channels', 'Movies', 'Clear Cache']
    return MAINLIST


def get_langs():
    """
    Get the list of languages.
    https://apiv2.voot.com/wsv_2_3/media/assetDetails.json?tabId=movieDetail&subTabId=allMovies&rowId=733&language=Hindi%2CEnglish&sortId=mostPopular&type=more&limit=15&offSet=0
    :return: list
    """
    languages = ['Hindi', 'Tamil', 'Telugu', 'Kannada', 'Bengali', 'Gujarati']
    langs = []
    for item in languages:
        url = '{0}media/assetDetails.json?tabId=movieDetail&subTabId=allMovies&rowId=733&language={1}&{2}&type=more&limit=8&offSet=0'.format(apiUrl, item, msort)
        jd = requests.get(url, headers=headers).json()
        tcount = jd['assets'][0]['totalItems']
        langs.append((item, tcount))

    return langs


def get_channels():
    """
    Get the list of channels.
    :return: list
    """
    channels = []
    url = '{0}media/assetDetails.json?tabId=channels&subTabId=channelList&{1}'.format(apiUrl, sortID)
    jd = requests.get(url, headers=headers).json()
    items = jd['assets'][0]['items']
    for item in items:
        title = item.get('channelName')
        tcount = item.get('total_item_count')
        icon = item.get('imgURL')
        sbu = item.get('sbu')
        channels.append((title, icon, sbu, tcount))

    return channels


def get_shows(channel, totals):
    """
    Get the list of shows.
    :return: list
    """
    shows = []
    offSet = 0
    totals = int(totals)
    if totals > 50:
        finalpg = totals % 50
        pages = int(totals / 50)
    else:
        finalpg = totals
        pages = 0

    while pages > 0:
        url = '{0}media/assetDetails.json?tabId=channels&subTabId=allShows&rowId=747&sbu={1}&{2}&type=more&limit=50&offSet={3}'.format(apiUrl, channel, sortID, offSet)
        jd = requests.get(url, headers=headers).json()
        items = jd['assets'][0]['items']
        for item in items:
            title = item.get('title')
            tcount = item.get('episodesCount')
            sid = item.get('mId')
            icon = item.get('imgURL')
            labels = {'title': title,
                      'genre': item.get('genre'),
                      'season': item.get('season'),
                      'plot': item.get('desc'),
                      'mediatype': 'tvshow',
                      'year': item.get('yearofRelease')}
            shows.append((title, icon, sid, tcount, labels))
        pages -= 1
        offSet += 50

    if finalpg > 0:
        url = '{0}media/assetDetails.json?tabId=channels&subTabId=allShows&rowId=747&sbu={1}&{2}&type=more&limit={3}&offSet={4}'.format(apiUrl, channel, sortID, finalpg, offSet)
        jd = requests.get(url, headers=headers).json()
        items = jd['assets'][0]['items']
        for item in items:
            title = item.get('title')
            tcount = item.get('episodesCount')
            sid = item.get('mId')
            icon = item.get('imgURL')
            labels = {'title': title,
                      'genre': item.get('genre'),
                      'season': item.get('season'),
                      'plot': item.get('desc'),
                      'mediatype': 'tvshow',
                      'year': item.get('yearofRelease')}
            shows.append((title, icon, sid, tcount, labels))

    return shows


def get_movies(lang, offSet, totals):
    """
    Get the list of movies.
    :return: list
    """
    movies = []
    totals = int(totals)
    offSet = int(offSet)
    finalpg = True
    itemsLeft = totals - (offSet + 1) * 50
    if itemsLeft > 0:
        finalpg = False
        pages = int(math.ceil(totals / 50.0))

    url = '{0}media/assetDetails.json?tabId=movieDetail&subTabId=allMovies&rowId=733&language={1}&{2}&type=more&limit=50&offSet={3}'.format(apiUrl, lang, msort, offSet * 50)
    jd = requests.get(url, headers=headers).json()
    items = jd['assets'][0]['items']
    for item in items:
        title = item.get('title')
        mid = item.get('mId')
        icon = item.get('imgURL')
        labels = {'title': title,
                  'genre': item.get('genre'),
                  'plot': item.get('desc'),
                  'duration': int(int(item.get('duration', '0')) / 1000),
                  'mediatype': 'movie',
                  'year': item.get('yearofRelease')
                  }
        movies.append((title, icon, mid, labels))

    if not finalpg:
        offSet += 1
        title = 'Next Page.. (Currently in Page {0} of {1})'.format(offSet, pages)
        labels = {}
        movies.append((title, _icon, offSet, labels))

    return movies


def get_episodes(show, offSet, totals):
    """
    Get the list of episodes.
    :return: list
    """
    episodes = []

    url = '{0}media/assetDetails.json?tabId=showDetail&subTabId=allEpisodes&rowId=792&tvSeriesId={1}&sortId=newestFirst&type=more&limit=4&offSet=0'.format(apiUrl, show)
    jd = requests.get(url, headers=headers).json()
    try:
        totals = int(jd['assets'][0]['totalItems'])
    except:
        totals = 0

    offSet = int(offSet)
    finalpg = True
    itemsLeft = totals - (offSet + 1) * 50
    if itemsLeft > 0:
        finalpg = False
        pages = int(math.ceil(totals / 50.0))

    url = '{0}media/assetDetails.json?tabId=showDetail&subTabId=allEpisodes&rowId=792&tvSeriesId={1}&sortId=newestFirst&type=more&limit=50&offSet={2}'.format(apiUrl, show, offSet)
    jd = requests.get(url, headers=headers).json()

    try:
        items = jd['assets'][0]['items']
    except:
        items = {}

    for item in items:
        title = item.get('title')
        eid = item.get('mId')
        icon = item.get('imgURL') if item.get('imgURL') else _icon

        labels = {'title': title,
                  'genre': item['genre'],
                  'plot': item['desc'],
                  'duration': int(int(item['duration']) / 1000),
                  'tvshowtitle': item['refSeriesTitle'],
                  'mediatype': 'episode',
                  'episode': item.get('episodeNo'),
                  'season': item.get('season')
                  }
        title = 'E{0} {1}'.format(labels.get('episode'), title) if labels.get('episode') else title
        title = 'S{0:02d}{1}'.format(int(labels.get('season')), title) if labels.get('season') else title
        td = item.get('telecastDate')
        if td:
            labels.update({'aired': td[:4] + '-' + td[4:6] + '-' + td[6:]})
        episodes.append((title, icon, eid, labels))

    if not finalpg:
        offSet += 1
        title = 'Next Page.. (Currently in Page {0} of {1})'.format(offSet, pages)
        labels = {}
        episodes.append((title, _icon, offSet, labels))

    return episodes


def list_top():
    """
    Create the list of countries in the Kodi interface.
    """
    items = get_top()
    listing = []
    for item in items:
        list_item = xbmcgui.ListItem(label=item)
        list_item.setInfo('video', {'title': item, 'genre': item})
        list_item.setArt({'poster': _icon,
                          'icon': _icon,
                          'thumb': _icon,
                          'fanart': _fanart})
        url = '{0}?action=list_item&item={1}'.format(_url, item)
        is_folder = True
        listing.append((url, list_item, is_folder))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.endOfDirectory(_handle)


def list_langs():
    """
    Create the list of countries in the Kodi interface.
    """
    langs = cache.cacheFunction(get_langs)
    listing = []
    for lang, tcount in langs:
        list_item = xbmcgui.ListItem(label='[COLOR yellow]{0}  [COLOR cyan]({1} movies)[/COLOR]'.format(lang, tcount))
        list_item.setInfo('video', {'title': lang, 'genre': lang})
        list_item.setArt({'poster': _icon,
                          'icon': _icon,
                          'thumb': _icon,
                          'fanart': _fanart})
        url = '{0}?action=list_movies&lang={1}&offSet=0&totals={2}'.format(_url, lang, tcount)
        is_folder = True
        listing.append((url, list_item, is_folder))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.endOfDirectory(_handle)


def list_channels():
    """
    Create the list of countries in the Kodi interface.
    """
    channels = cache.cacheFunction(get_channels)
    listing = []
    for title, icon, sbu, tcount in channels:
        list_item = xbmcgui.ListItem(label='[COLOR yellow]{0}  [COLOR cyan]({1} shows)[/COLOR]'.format(title, tcount))
        list_item.setArt({'poster': icon,
                          'icon': icon,
                          'thumb': icon,
                          'fanart': _fanart})
        list_item.setInfo('video', {'title': title})
        url = '{0}?action=list_channel&channel={1}&totals={2}'.format(_url, sbu, tcount)
        is_folder = True
        listing.append((url, list_item, is_folder))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.setContent(_handle, 'videos')
    xbmcplugin.endOfDirectory(_handle)


def list_shows(channel, totals):
    """
    Create the list of channels in the Kodi interface.
    """
    shows = cache.cacheFunction(get_shows, channel, totals)
    listing = []
    for title, icon, sid, tcount, labels in shows:
        list_item = xbmcgui.ListItem(label='[COLOR yellow]{0}  [COLOR cyan]({1} episodes)[/COLOR]'.format(title, tcount))
        list_item.setArt({'poster': icon,
                          'icon': icon,
                          'thumb': icon,
                          'fanart': icon})
        list_item.setInfo('video', labels)
        url = '{0}?action=list_show&show={1}&offSet=0&totals={2}&icon={3}'.format(_url, sid, tcount, icon)
        is_folder = True
        listing.append((url, list_item, is_folder))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.setContent(_handle, 'tvshows')
    xbmcplugin.endOfDirectory(_handle)


def list_movies(lang, offSet, totals):
    """
    Create the list of episodes in the Kodi interface.
    """
    movies = cache.cacheFunction(get_movies, lang, offSet, totals)
    listing = []
    for title, icon, mid, labels in movies:
        list_item = xbmcgui.ListItem(label=title)
        list_item.setArt({'poster': icon,
                          'icon': icon,
                          'thumb': icon,
                          'fanart': icon})
        list_item.setInfo('video', labels)
        is_folder = False
        if 'Next Page' not in title:
            list_item.setProperty('IsPlayable', 'true')
            url = '{0}?action=play&video={1}'.format(_url, mid)
        else:
            url = '{0}?action=list_movies&lang={1}&offSet={2}&totals={3}'.format(_url, lang, mid, totals)
            is_folder = True
        listing.append((url, list_item, is_folder))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.setContent(_handle, 'movies')
    xbmcplugin.endOfDirectory(_handle)


def list_episodes(show, offSet, totals, sicon):
    """
    Create the list of episodes in the Kodi interface.
    """
    episodes = cache.cacheFunction(get_episodes, show, offSet, totals)
    listing = []
    for title, icon, eid, labels in episodes:
        list_item = xbmcgui.ListItem(label=title)
        list_item.setArt({'poster': icon,
                          'icon': icon,
                          'thumb': icon,
                          'fanart': sicon})
        list_item.setInfo('video', labels)
        is_folder = False
        if 'Next Page' not in title:
            list_item.setProperty('IsPlayable', 'true')
            url = '{0}?action=play&video={1}'.format(_url, eid)
        else:
            url = url = '{0}?action=list_show&show={1}&offSet={2}&totals={3}&icon={4}'.format(_url, show, eid, totals, sicon)
            is_folder = True
        listing.append((url, list_item, is_folder))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.setContent(_handle, 'episodes')
    xbmcplugin.endOfDirectory(_handle)


def play_video(path, strqual):
    """
    Play a video by the provided path.

    :param path: str
    """
    # Create a playable item with a path to play.
    play_item = xbmcgui.ListItem(path=path)
    vid_link = play_item.getfilename()
    url = '{0}playBack.json?mediaId={1}'.format(apiUrl, vid_link)
    jd = requests.get(url, headers=headers).json()
    files = jd['assets'][0]['assets'][0]['items'][0]['files']
    if strqual not in str(files):
        strqual = 'HLS_PremiumHD'
    for file in files:
        if file.get('Format') == strqual:
            stream_url = file.get('URL') + '|User-Agent=playkit/android-3.4.5 com.tv.v18.viola/2.1.42 (Linux;Android 5.1.1) ExoPlayerLib/2.7.0'
            break
    if _settings('EnableIP') == 'true':
        stream_url += '&X-Forwarded-For={0}'.format(_settings('ipaddress'))
    play_item.setPath(stream_url)
    # Pass the item to the Kodi player.
    xbmcplugin.setResolvedUrl(_handle, True, listitem=play_item)
